@@ -5,12 +5,27 @@ from django.utils import simplejson as json
 from django.core import exceptions
 from django.core.serializers.json import DjangoJSONEncoder
 from django.utils.translation import ugettext as _
+from django.core.exceptions import ImproperlyConfigured
 
+import re
 import decimal
 from datetime import datetime
-from dateutil import parser as date_parser
+try:
+    from dateutil import parser as date_parser
+except ImportError:
+    raise ImproperlyConfigured('The "dateutil" library is required and was not found.')
+
+try:
+    JSON_DECODE_ERROR = json.JSONDecodeError # simplejson
+except AttributeError:
+    JSON_DECODE_ERROR = ValueError # other
+
+TIME_RE = re.compile(r'^\d{2}:\d{2}:\d{2}')
+DATE_RE = re.compile(r'^\d{4}-\d{2}-\d{2}(?!T)')
+DATETIME_RE = re.compile(r'^\d{4}-\d{2}-\d{2}T')
 
 class JSONDecoder(json.JSONDecoder):
+    """ Recursive JSON to Python deserialization. """
 
     _recursable_types = [str, unicode, list, dict]
 
@@ -31,25 +46,24 @@ class JSONDecoder(json.JSONDecoder):
                     obj[key] = self.decode(value, recurse=True)
         elif isinstance(obj, basestring):
             try:
-                return decimal.Decimal(obj) # first since dateutil will consume this
+                return decimal.Decimal(obj)
             except decimal.InvalidOperation:
                 pass
-            try:
-                return datetime.strptime(obj, '%H:%M:%S.%f').time() # with microsecond
-            except ValueError:
-                pass
-            try:
-                return datetime.strptime(obj, '%H:%M:%S').time() # without microsecond
-            except ValueError:
-                pass
-            try:
-                return datetime.strptime(obj, '%Y-%m-%d').date() # simple date
-            except ValueError:
-                pass
-            try:
-                return date_parser.parse(obj) # supports multiple formats
-            except ValueError:
-                pass
+            if TIME_RE.match(obj):
+                try:
+                    return date_parser.parse(obj).time()
+                except ValueError:
+                    pass
+            if DATE_RE.match(obj):
+                try:
+                    return date_parser.parse(obj).date()
+                except ValueError:
+                    pass
+            if DATETIME_RE.match(obj):
+                try:
+                    return date_parser.parse(obj)
+                except ValueError:
+                    pass
         return obj
 
 class JSONField(models.TextField):
@@ -59,12 +73,13 @@ class JSONField(models.TextField):
 
     description = 'JSON object'
 
-    default_error_messages = {
-        'invalid': _(u'Enter a valid JSON object')
-    }
-
     def __init__(self, *args, **kwargs):
+        self.default_error_messages = {
+            'invalid': _(u'Enter a valid JSON object')
+        }
         self._db_type = kwargs.pop('db_type', None)
+        self.simple_formfield = kwargs.pop('simple_formfield', False)
+
         encoder = kwargs.pop('encoder', DjangoJSONEncoder)
         decoder = kwargs.pop('decoder', JSONDecoder)
         encoder_kwargs = kwargs.pop('encoder_kwargs', {})
@@ -75,8 +90,10 @@ class JSONField(models.TextField):
             decoder_kwargs.update({'cls':decoder})
         self.encoder_kwargs = encoder_kwargs
         self.decoder_kwargs = decoder_kwargs
-        kwargs['default'] = kwargs.get('default', {})
+
+        kwargs['default'] = kwargs.get('default', 'null')
         kwargs['help_text'] = kwargs.get('help_text', self.default_error_messages['invalid'])
+
         super(JSONField, self).__init__(*args, **kwargs)
 
     def db_type(self, connection):
@@ -85,17 +102,17 @@ class JSONField(models.TextField):
         return super(JSONField, self).db_type(connection)
 
     def to_python(self, value):
-        if not value:
+        if value is None: # allow blank objects
             return None
         if isinstance(value, basestring):
             try:
                 value = json.loads(value, **self.decoder_kwargs)
-            except json.JSONDecodeError:
+            except JSON_DECODE_ERROR:
                 pass
         return value
 
     def get_db_prep_value(self, value, *args, **kwargs):
-        if value is None:
+        if self.null and value is None and not kwargs.get('force'):
             return None
         return json.dumps(value, **self.encoder_kwargs)
 
@@ -108,12 +125,24 @@ class JSONField(models.TextField):
     def formfield(self, **kwargs):
         defaults = {
             'form_class': kwargs.get('form_class', JSONFormField),
+            'simple': self.simple_formfield,
             'encoder_kwargs': self.encoder_kwargs,
             'decoder_kwargs': self.decoder_kwargs,
         }
         defaults.update(kwargs)
         return super(JSONField, self).formfield(**defaults)
 
+    def contribute_to_class(self, cls, name):
+        super(JSONField, self).contribute_to_class(cls, name)
+
+        def get_json(model_instance):
+            return self.get_db_prep_value(getattr(model_instance, self.attname, None), force=True)
+        setattr(cls, 'get_%s_json' % self.name, get_json)
+
+        def set_json(model_instance, value):
+            return setattr(model_instance, self.attname, self.to_python(value))
+        setattr(cls, 'set_%s_json' % self.name, set_json)
+
 try:
     # add support for South migrations
     from south.modelsinspector import add_introspection_rules
